package util.test;

import java.math.BigInteger;
import java.util.*;

import util.*;

/**
 * Eine beliebig grosse Binrzahl zum
 * Erzeugen von Testfllen.
 * 
 * Es wird ein Array mit allen
 * Einzel-Test-Fllen bergeben.
 * 
 * In jedem Laufschritt wird der
 * Binrzhler erhht und die
 * zutreffenden Einzel-Test-Flle
 * (Positions-Bit == true)
 * ausgefiltert.
 * 
 * Dadurch werden alle mglichen Kombinationen
 * (bei 4 Einzel-Test-Fllen 16 mgliche Kombinationen)
 * zurckgeliefert.
 * 
 * TODO evtl. Umstellung auf Iterator-Prinzip: hasNext() und next()
 * 
 * @author Heiner Kcker
 */
public final class BigBinary
{
    /**
     * Array mit den einzelnen Bits
     * 
     * TODO Abbildung auf int-Array mit je 32 Bit
     */
    private final boolean[] boolArr;
    
    /**
     * berlauf-Merker
     */
    private boolean carry;
    
    /**
     * Constructor.
     *
     * @param pBitCount
     */
    public BigBinary(
            final int pBitCount )
    {
        this.boolArr = new boolean[ pBitCount ];
    }

    /**
     * @return the carry
     */
    public boolean carry()
    {
        return this.carry;
    }
    
    /**
     * 
     * @param pBitPos
     * @return
     */
    public boolean get(
            final int pBitPos )
    {
        return this.boolArr[ pBitPos ];
    }

    /**
     * set bit to true.
     * 
     * @param pBitPos
     */
    public void set(
            final int pBitPos )
    {
        this.boolArr[ pBitPos ] = true;
    }

    /**
     * set bit to given value.
     * 
     * @param pBitPos
     */
    public void set(
            final int pBitPos ,
            final boolean pValue )
    {
        this.boolArr[ pBitPos ] = pValue;
    }

    /**
     * reset bit to false.
     * 
     * @param pBitPos
     */
    public void reset(
            final int pBitPos )
    {
        this.boolArr[ pBitPos ] = false;
    }

    /**
     * negate bit.
     * 
     * @param pBitPos
     */
    public void negate(
            final int pBitPos )
    {
        this.boolArr[ pBitPos ] = ! this.boolArr[ pBitPos ];
    }

    /**
     * Clear this.
     */
    public void clear()
    {
        Arrays.fill( this.boolArr, false );
        this.carry = false;
    }

    /**
     * Um eins erhhen.
     */
    public void increment()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        for (int i = 0; i < this.boolArr.length; i++)
        {
            if ( ! this.boolArr[ i ] )
            {
                this.boolArr[ i ] = true;
                // alle Stellen erledigt
                return;
            }
            else
            {
                // Zurcksetzen und bertrag auf die nchste Stelle
                this.boolArr[ i ] = false;
            }
        }
        // berlauf
        this.carry = true;
    }
    
    /**
     * Zurckgeben des inneren Wertes als long.
     * Achtung, keine schutz gegen Zahlenberlauf.
     * 
     * @return
     */
    public long longValue()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        
        long potenz = 1;
        long retVal = 0;
        
        for (int i = 0; i < this.boolArr.length; i++)
        {
            if ( this.boolArr[ i ] )
            {
                retVal += potenz;
            }
            potenz *= 2;
        }
        return retVal;
    }

    /**
     * 
     * @return
     */
    public BigInteger toBigInteger()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        return new BigInteger( this.binStr() , 2 );
    }

    /**
     * Filtere das bergebene Object-Array
     * so aus, dass nur die Elemente zurckgegeben
     * werden, an deren Position dieses {@link BigBinary}
     * true ist.
     * 
     * Das zurckgegebene Arra enthlt die
     * aktuellen Test-Permutationen.
     * 
     * @param pObjArr
     * @return
     */
    public Object[] filter(
            final Object[] pObjArr )
    {
        if ( pObjArr.length != this.boolArr.length  )
        {
            throw new IllegalArgumentException( "Lngendiferenz pObjArr: " + pObjArr.length + " this.boolArr.length: " + this.boolArr.length );
        }
        final ArrayList list = new ArrayList( pObjArr.length );
        for (int i = this.boolArr.length - 1; i >= 0 ; i-- )
        {
            if ( this.boolArr[ i ] )
            {
                list.add( pObjArr[ i ] );
            }
        }
        return list.toArray( //--
                // TODO mal in eine Util-Klasse auslagern
                (Object[]) //--
                java.lang.reflect.Array.newInstance( //--
                        pObjArr.getClass().getComponentType() , list.size() ) );
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return 
            StringUtil.getShortClassName( this ) + " "
            // TODO mache eine Methode toBigInteger zum vermeiden eines Zahlenberlaufes
            + this.longValue() + " '" + this.binStr() + "' length: " + this.boolArr.length + " carry: " + this.carry ;
    }

    /**
     * Zurckgeben als String mit Nullen und Einsen.
     * 
     * @return
     */
    public String binStr()
    {
        final UnSynchStringBuffer strBuff = new UnSynchStringBuffer( this.boolArr.length );
        for (int i = this.boolArr.length - 1; i >= 0 ; i-- )
        {
            strBuff.append( this.boolArr[ i ] ? "1" : "0" );
        }
        return strBuff.toString();
    }

    /**
     * Test.
     * 
     * @param args
     */
    public static void main(String[] args)
    {
        final int testCount = 4;
        
        final String[] testStrArr = new String[ testCount ];
        for (int i = 0; i < testStrArr.length; i++)
        {
            testStrArr[i] = "Test" + i;
        }
        
        final BigBinary bigBinary = new BigBinary( testCount );
        while ( ! bigBinary.carry() )
        {
            System.out.println( bigBinary );
            System.out.println( Arrays.asList(
                    bigBinary.filter(
                            testStrArr )) );

            bigBinary.increment();
        }
    }

}
